@@ -7,15 +7,15 @@ use wgpu::*;
 
 use crate::draw::{Command as DrawCommand, DrawList, Update, Vertex};
 use crate::layout::Rectangle;
-use crate::loader::Loader;
-use crate::{Command, EventLoop, UpdateModel, Model};
+use crate::style::Style;
+use crate::Component;
 use std::num::NonZeroU32;
 use wgpu::util::DeviceExt;
 
 /// Wrapper for [`Ui`](../../struct.Ui.html) that adds wgpu rendering.
 /// Requires the "wgpu" feature.
-pub struct Ui<M: Model + for<'a> UpdateModel<'a>, E: EventLoop<Command<<M as Model>::Message>>, L: Loader> {
-    inner: crate::Ui<M, E, L>,
+pub struct Ui<C: 'static + Component> {
+    inner: crate::Ui<C>,
     pipeline: RenderPipeline,
     bind_group_layout: BindGroupLayout,
     sampler: Sampler,
@@ -29,32 +29,38 @@ struct TextureEntry {
     bind_group: BindGroup,
 }
 
-impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Model>::Message>>, L: 'static + Loader> Ui<M, E, L> {
-    /// Constructs a new `Ui` using the default style.
-    /// This is not recommended as the default style is very empty and only renders white text.
-    pub fn new(
-        model: M,
-        event_loop: E,
-        loader: L,
+impl<C: Component> Ui<C> {
+    /// Constructs a new `Ui`. Returns an error if the style fails to load.
+    pub fn new<S, E>(
+        root_component: C,
         viewport: Rectangle,
+        hidpi_scale: f32,
+        style: S,
         format: wgpu::TextureFormat,
         device: &Device,
-    ) -> Self {
-        Self::new_inner(crate::Ui::new(model, event_loop, loader, viewport), format, device)
+    ) -> anyhow::Result<Self>
+    where
+        S: TryInto<Style, Error = E>,
+        anyhow::Error: From<E>,
+    {
+        Ok(Self::new_inner(
+            crate::Ui::new(root_component, viewport, hidpi_scale, style)?,
+            format,
+            device,
+        ))
     }
 
-    fn new_inner(inner: crate::Ui<M, E, L>, format: wgpu::TextureFormat, device: &Device) -> Self {
+    fn new_inner(inner: crate::Ui<C>, format: wgpu::TextureFormat, device: &Device) -> Self {
         let shader_module = device.create_shader_module(&ShaderModuleDescriptor {
             label: Some("wgpu.wgsl"),
             source: wgpu::ShaderSource::Wgsl(include_str!("wgpu.wgsl").into()),
-            flags: ShaderFlags::VALIDATION,
         });
         let bind_group_layout = device.create_bind_group_layout(&wgpu::BindGroupLayoutDescriptor {
             label: None,
             entries: &[
                 wgpu::BindGroupLayoutEntry {
                     binding: 0,
-                    visibility: wgpu::ShaderStage::FRAGMENT,
+                    visibility: wgpu::ShaderStages::FRAGMENT,
                     ty: wgpu::BindingType::Texture {
                         sample_type: wgpu::TextureSampleType::Float { filterable: true },
                         multisampled: false,
@@ -64,7 +70,7 @@ impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Mo
                 },
                 wgpu::BindGroupLayoutEntry {
                     binding: 1,
-                    visibility: wgpu::ShaderStage::FRAGMENT,
+                    visibility: wgpu::ShaderStages::FRAGMENT,
                     ty: wgpu::BindingType::Sampler {
                         filtering: true,
                         comparison: false,
@@ -86,7 +92,7 @@ impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Mo
                 entry_point: "vs_main",
                 buffers: &[wgpu::VertexBufferLayout {
                     array_stride: std::mem::size_of::<Vertex>() as u64,
-                    step_mode: wgpu::InputStepMode::Vertex,
+                    step_mode: wgpu::VertexStepMode::Vertex,
                     attributes: &[
                         wgpu::VertexAttribute {
                             format: VertexFormat::Float32x2,
@@ -123,7 +129,7 @@ impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Mo
                 targets: &[wgpu::ColorTargetState {
                     format,
                     blend: Some(wgpu::BlendState::ALPHA_BLENDING),
-                    write_mask: wgpu::ColorWrite::ALL,
+                    write_mask: wgpu::ColorWrites::ALL,
                 }],
             }),
         });
@@ -154,7 +160,9 @@ impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Mo
         }
     }
 
-    /// Generate a [`DrawList`](draw/struct.DrawList.html) for the view.
+    /// Draw the ui to a `RenderPass`.
+    /// The `device` must be the same as the one passed to [`new()`](#method.new).
+    /// The `render_pass` render target must be compatible with the `texture_format` passed to [`new`](#method.new).
     pub fn draw<'a>(&'a mut self, device: &Device, queue: &Queue, render_pass: &mut RenderPass<'a>) {
         if self.inner.needs_redraw() {
             let DrawList {
@@ -190,7 +198,7 @@ impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Mo
                                         sample_count: 1,
                                         dimension: wgpu::TextureDimension::D2,
                                         format: wgpu::TextureFormat::Rgba8Unorm,
-                                        usage: wgpu::TextureUsage::SAMPLED | wgpu::TextureUsage::COPY_DST,
+                                        usage: wgpu::TextureUsages::TEXTURE_BINDING | wgpu::TextureUsages::COPY_DST,
                                     };
                                     let texture = if data.is_empty() {
                                         device.create_texture(&texture_desc)
@@ -237,7 +245,7 @@ impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Mo
                                     let staging = device.create_buffer_init(&wgpu::util::BufferInitDescriptor {
                                         label: None,
                                         contents: data.as_slice(),
-                                        usage: wgpu::BufferUsage::COPY_SRC,
+                                        usage: wgpu::BufferUsages::COPY_SRC,
                                     });
                                     cmd.copy_buffer_to_texture(
                                         wgpu::ImageCopyBuffer {
@@ -249,13 +257,14 @@ impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Mo
                                             },
                                         },
                                         wgpu::ImageCopyTexture {
-                                            texture: &texture,
+                                            texture,
                                             mip_level: 0,
                                             origin: wgpu::Origin3d {
                                                 x: offset[0],
                                                 y: offset[1],
                                                 z: 0,
                                             },
+                                            aspect: wgpu::TextureAspect::All,
                                         },
                                         wgpu::Extent3d {
                                             width: size[0],
@@ -276,7 +285,7 @@ impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Mo
                     .replace(device.create_buffer_init(&wgpu::util::BufferInitDescriptor {
                         label: None,
                         contents: vertices.as_bytes(),
-                        usage: wgpu::BufferUsage::VERTEX,
+                        usage: wgpu::BufferUsages::VERTEX,
                     }));
             }
         }
@@ -310,15 +319,15 @@ impl<M: Model + for<'a> UpdateModel<'a>, E: 'static + EventLoop<Command<<M as Mo
     }
 }
 
-impl<M: Model + for<'a> UpdateModel<'a>, E: EventLoop<Command<<M as Model>::Message>>, L: Loader> Deref for Ui<M, E, L> {
-    type Target = crate::Ui<M, E, L>;
+impl<C: Component> Deref for Ui<C> {
+    type Target = crate::Ui<C>;
 
     fn deref(&self) -> &Self::Target {
         &self.inner
     }
 }
 
-impl<M: Model + for<'a> UpdateModel<'a>, E: EventLoop<Command<<M as Model>::Message>>, L: Loader> DerefMut for Ui<M, E, L> {
+impl<C: Component> DerefMut for Ui<C> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         &mut self.inner
     }
