@@ -1,45 +1,66 @@
 use winit::{
     event::{Event, WindowEvent},
-    event_loop::{ControlFlow, EventLoop, EventLoopProxy},
+    event_loop::{ControlFlow, EventLoop},
     window::{Window, WindowBuilder},
 };
 
-use crate::loader::Loader;
 use crate::prelude::*;
+use std::sync::{Arc, Mutex};
+use std::task::Wake;
+use winit::event_loop::EventLoopProxy;
 
 /// Sandbox for quick prototyping of pixel widgets applications
-pub struct Sandbox<M: Model + for<'a> UpdateModel<'a>, L: 'static + Loader> {
+pub struct Sandbox<M: 'static + Component> {
     /// The `Ui` being used in the sandbox
-    pub ui: crate::backend::wgpu::Ui<M, EventLoopProxy<Command<<M as Model>::Message>>, L>,
-    event_loop: Option<EventLoop<Command<<M as Model>::Message>>>,
+    pub ui: crate::backend::wgpu::Ui<M>,
+    event_loop: Option<EventLoop<PollUi>>,
     surface: wgpu::Surface,
     #[allow(unused)]
     adapter: wgpu::Adapter,
     device: wgpu::Device,
     queue: wgpu::Queue,
-    swap_chain: wgpu::SwapChain,
-    sc_desc: wgpu::SwapChainDescriptor,
+    surface_config: wgpu::SurfaceConfiguration,
     window: Window,
 }
 
-impl<T, L> Sandbox<T, L>
+#[derive(Clone)]
+struct PollUi;
+
+struct Waker<T: 'static> {
+    message: T,
+    event_loop: Mutex<EventLoopProxy<T>>,
+}
+
+impl<T: 'static + Clone> Wake for Waker<T> {
+    fn wake(self: Arc<Self>) {
+        self.event_loop.lock().unwrap().send_event(self.message.clone()).ok();
+    }
+}
+
+impl<T> Sandbox<T>
 where
-    T: Model + for<'a> UpdateModel<'a, State = ()>,
-    L: 'static + Loader,
+    T: 'static + Component,
 {
-    /// Construct a new `Sandbox`
-    pub async fn new(model: T, loader: L, builder: WindowBuilder) -> Self {
-        let event_loop = EventLoop::<Command<T::Message>>::with_user_event();
-        let window = builder.build(&event_loop).unwrap();
+    /// Construct a new `Sandbox` with a root component, style and window builder.
+    /// The `Sandbox` will finish building the window and setup the graphics.
+    /// To start the main event loop, call [`run()`](#method.run) on the result.
+    pub async fn new<S, E>(root_component: T, style: S, window: WindowBuilder) -> anyhow::Result<Self>
+    where
+        S: TryInto<Style, Error = E>,
+        anyhow::Error: From<E>,
+    {
+        let event_loop = EventLoop::with_user_event();
+        let window = window.build(&event_loop).unwrap();
         let size = window.inner_size();
 
         let swapchain_format = wgpu::TextureFormat::Bgra8Unorm;
 
-        let instance = wgpu::Instance::new(wgpu::BackendBit::PRIMARY);
+        let instance = wgpu::Instance::new(wgpu::Backends::PRIMARY);
         let surface = unsafe { instance.create_surface(&window) };
         let adapter = instance
             .request_adapter(&wgpu::RequestAdapterOptions {
                 power_preference: wgpu::PowerPreference::LowPower,
+                force_fallback_adapter: false,
                 compatible_surface: Some(&surface),
             })
             .await
@@ -59,63 +80,85 @@ where
             .await
             .expect("Failed retrieve device and queue");
 
-        let sc_desc = wgpu::SwapChainDescriptor {
-            usage: wgpu::TextureUsage::RENDER_ATTACHMENT,
+        let surface_config = wgpu::SurfaceConfiguration {
+            usage: wgpu::TextureUsages::RENDER_ATTACHMENT,
             format: swapchain_format,
             width: size.width,
             height: size.height,
             present_mode: wgpu::PresentMode::Mailbox,
         };
 
-        let swap_chain = device.create_swap_chain(&surface, &sc_desc);
+        surface.configure(&device, &surface_config);
 
         let ui = crate::backend::wgpu::Ui::new(
-            model,
-            event_loop.create_proxy(),
-            loader,
+            root_component,
             Rectangle::from_wh(size.width as f32, size.height as f32),
+            window.scale_factor() as f32,
+            style,
             swapchain_format,
             &device,
-        );
+        )?;
 
-        Sandbox {
+        Ok(Sandbox {
             ui,
             event_loop: Some(event_loop),
             surface,
             adapter,
             device,
             queue,
-            swap_chain,
-            sc_desc,
+            surface_config,
             window,
-        }
+        })
+    }
+
+    /// Update the root component with a message.
+    /// Returns any output messages from the root component.
+    pub fn update(&mut self, message: T::Message) -> Vec<T::Output> {
+        let waker = self
+            .event_loop
+            .as_ref()
+            .map(|event_loop| {
+                std::task::Waker::from(Arc::new(Waker {
+                    message: PollUi,
+                    event_loop: Mutex::new(event_loop.create_proxy()),
+                }))
+            })
+            .unwrap();
+
+        self.ui.update_and_poll(message, waker)
     }
 
     /// Run the application
     pub async fn run(mut self) {
         let event_loop = self.event_loop.take().unwrap();
+        let waker = std::task::Waker::from(Arc::new(Waker {
+            message: PollUi,
+            event_loop: Mutex::new(event_loop.create_proxy()),
+        }));
+
         event_loop.run(move |event, _, control_flow| {
             *control_flow = ControlFlow::Wait;
             match event {
-                Event::UserEvent(command) => {
-                    self.ui.command(command, &mut ());
+                Event::UserEvent(_) => {
+                    self.ui.poll(waker.clone());
                 }
                 Event::WindowEvent {
                     event: WindowEvent::Resized(size),
                     ..
                 } => {
                     // Recreate the swap chain with the new size
-                    self.sc_desc.width = size.width;
-                    self.sc_desc.height = size.height;
-                    self.swap_chain = self.device.create_swap_chain(&self.surface, &self.sc_desc);
+                    self.surface_config.width = size.width;
+                    self.surface_config.height = size.height;
+                    self.surface.configure(&self.device, &self.surface_config);
                     self.ui
                         .resize(Rectangle::from_wh(size.width as f32, size.height as f32));
                 }
                 Event::RedrawRequested(_) => {
                     let frame = self
-                        .swap_chain
-                        .get_current_frame()
+                        .surface
+                        .get_current_texture()
                         .expect("Failed to acquire next swap chain texture");
+                    let view = frame.texture.create_view(&wgpu::TextureViewDescriptor::default());
                     let mut encoder = self
                         .device
                         .create_command_encoder(&wgpu::CommandEncoderDescriptor { label: None });
@@ -123,7 +166,7 @@ where
                         let mut pass = encoder.begin_render_pass(&wgpu::RenderPassDescriptor {
                             label: None,
                             color_attachments: &[wgpu::RenderPassColorAttachment {
-                                view: &frame.output.view,
+                                view: &view,
                                 resolve_target: None,
                                 ops: wgpu::Operations {
                                     load: wgpu::LoadOp::Clear(wgpu::Color::BLACK),
@@ -137,6 +180,7 @@ where
                     }
 
                     self.queue.submit(Some(encoder.finish()));
+                    frame.present();
                 }
                 Event::WindowEvent {
                     event: WindowEvent::CloseRequested,
@@ -144,7 +188,7 @@ where
                 } => *control_flow = ControlFlow::Exit,
                 other => {
                     if let Some(event) = crate::backend::winit::convert_event(other) {
-                        self.ui.event(event, &mut ());
+                        self.ui.handle_event_and_poll(event, waker.clone());
                     }
                 }
             }
