@@ -1,7 +1,8 @@
 use crate::draw::Primitive;
 use crate::layout::{Rectangle, Size};
-use crate::stylesheet::Stylesheet;
-use crate::widget::{IntoNode, Node, Widget};
+use crate::node::{GenericNode, IntoNode, Node};
+use crate::style::Stylesheet;
+use crate::widget::Widget;
 
 /// Dummy widget that has a custom widget name
 pub struct Dummy {
@@ -13,9 +14,25 @@ impl Dummy {
     pub fn new(widget: &'static str) -> Self {
         Self { widget }
     }
+
+    /// Sets the widget name for this dummy
+    pub fn widget(mut self, widget: &'static str) -> Self {
+        self.widget = widget;
+        self
+    }
+}
+
+impl Default for Dummy {
+    fn default() -> Self {
+        Dummy { widget: "" }
+    }
 }
 
 impl<'a, T: 'a> Widget<'a, T> for Dummy {
+    type State = ();
+
+    fn mount(&self) {}
+
     fn widget(&self) -> &'static str {
         self.widget
     }
@@ -24,19 +41,19 @@ impl<'a, T: 'a> Widget<'a, T> for Dummy {
         0
     }
 
-    fn visit_children(&mut self, _: &mut dyn FnMut(&mut Node<'a, T>)) {}
+    fn visit_children(&mut self, _: &mut dyn FnMut(&mut dyn GenericNode<'a, T>)) {}
 
-    fn size(&self, style: &Stylesheet) -> (Size, Size) {
+    fn size(&self, _: &(), style: &Stylesheet) -> (Size, Size) {
         (style.width, style.height)
     }
 
-    fn draw(&mut self, layout: Rectangle, _: Rectangle, style: &Stylesheet) -> Vec<Primitive<'a>> {
+    fn draw(&mut self, _: &mut (), layout: Rectangle, _: Rectangle, style: &Stylesheet) -> Vec<Primitive<'a>> {
         style.background.render(layout).into_iter().collect()
     }
 }
 
 impl<'a, T: 'a> IntoNode<'a, T> for Dummy {
     fn into_node(self) -> Node<'a, T> {
-        Node::new(self)
+        Node::from_widget(self)
     }
 }
