@@ -1,14 +1,29 @@
-use crate::draw::*;
-use crate::event::{Event, Key, Modifiers};
-use crate::layout::{Rectangle, Size};
-use crate::stylesheet::Stylesheet;
-use crate::text::{Text, TextWrap};
-use crate::widget::{Context, IntoNode, Node, Widget};
+use std::borrow::Cow;
+use std::time::Instant;
+
 #[cfg(feature = "clipboard")]
 use clipboard::{ClipboardContext, ClipboardProvider};
 use rusttype::Scale;
-use std::borrow::Cow;
-use std::time::Instant;
+use smallvec::smallvec;
+
+use crate::draw::*;
+use crate::event::{Event, Key, Modifiers};
+use crate::layout::{Rectangle, Size};
+use crate::node::{GenericNode, IntoNode, Node};
+use crate::style::{StyleState, Stylesheet};
+use crate::text::{Text, TextWrap};
+use crate::widget::{Context, Widget};
+
+use super::StateVec;
+
+#[cfg(target_os = "macos")]
+const BACKWARDS_DELETE: char = '\x7f';
+#[cfg(not(target_os = "macos"))]
+const BACKWARDS_DELETE: char = '\x08';
+#[cfg(target_os = "macos")]
+const FORWARD_DELETE: char = '\x08';
+#[cfg(not(target_os = "macos"))]
+const FORWARD_DELETE: char = '\x7f';
 
 /// State for [`Input`](struct.Input.html)
 pub struct State {
@@ -29,7 +44,6 @@ enum InnerState {
 /// Editable text input
 pub struct Input<'a, T, F, S> {
     placeholder: &'a str,
-    state: &'a mut State,
     password: bool,
     value: S,
     on_change: F,
@@ -44,10 +58,9 @@ where
     S: 'a + Send + AsRef<str>,
 {
     /// Construct a new `Input`
-    pub fn new(state: &'a mut State, placeholder: &'a str, value: S, on_change: F) -> Self {
+    pub fn new(placeholder: &'a str, value: S, on_change: F) -> Self {
         Input {
             placeholder,
-            state,
             password: false,
             value,
             on_change,
@@ -56,27 +69,50 @@ where
         }
     }
 
+    /// Sets the placeholder text, which is displayed when the input has no value.
+    pub fn placeholder(mut self, placeholder: &'a str) -> Self {
+        self.placeholder = placeholder;
+        self
+    }
+
     /// Construct a new `Input` that renders the text as dots, for passwords.
-    pub fn password(state: &'a mut State, placeholder: &'a str, value: S, on_change: F) -> Self {
+    pub fn password(mut self, password: bool) -> Self {
+        self.password = password;
+        self
+    }
+
+    /// Sets the current text value of the input.
+    pub fn val<N: AsRef<str>>(self, value: N) -> Input<'a, T, F, N> {
         Input {
-            placeholder,
-            state,
-            password: true,
+            placeholder: self.placeholder,
+            password: self.password,
             value,
+            on_change: self.on_change,
+            on_submit: self.on_submit,
+            trigger: self.trigger,
+        }
+    }
+
+    /// Sets the message to post when the text value should be changed to a new value.
+    pub fn on_change<N: Fn(String) -> T>(self, on_change: N) -> Input<'a, T, N, S> {
+        Input {
+            placeholder: self.placeholder,
+            password: self.password,
+            value: self.value,
             on_change,
-            on_submit: None,
-            trigger: None,
+            on_submit: self.on_submit,
+            trigger: self.trigger,
         }
     }
 
     /// Sets the message to post when the users submits using the enter key
-    pub fn with_on_submit(mut self, message: T) -> Self {
+    pub fn on_submit(mut self, message: T) -> Self {
         self.on_submit.replace(message);
         self
     }
 
-    /// Sets a key that will trigger input focus
-    pub fn with_trigger_key(mut self, key: Key) -> Self {
+    /// Sets a keyboard key that will trigger input focus
+    pub fn trigger_key(mut self, key: Key) -> Self {
         self.trigger.replace(key);
         self
     }
@@ -91,7 +127,7 @@ where
         }
     }
 
-    fn placeholder(&self, stylesheet: &Stylesheet) -> Text {
+    fn placeholder_text(&self, stylesheet: &Stylesheet) -> Text {
         Text {
             text: Cow::Borrowed(self.placeholder),
             font: stylesheet.font.clone(),
@@ -106,26 +142,53 @@ where
     }
 }
 
+impl<'a, T> Default for Input<'a, T, fn(String) -> T, &'static str> {
+    fn default() -> Self {
+        Self {
+            placeholder: "",
+            password: false,
+            value: "",
+            on_change: |_| panic!("on_change of `Input` must be set"),
+            on_submit: None,
+            trigger: None,
+        }
+    }
+}
+
 impl<'a, T, F, S> Widget<'a, T> for Input<'a, T, F, S>
 where
     T: 'a + Send,
     F: 'a + Send + Fn(String) -> T,
     S: 'a + Send + AsRef<str>,
 {
+    type State = State;
+
+    fn mount(&self) -> Self::State {
+        State::default()
+    }
+
     fn widget(&self) -> &'static str {
         "input"
     }
 
+    fn state(&self, state: &State) -> StateVec {
+        match state.inner {
+            InnerState::Dragging(_, _, _) => smallvec![StyleState::Focused],
+            InnerState::Focused(_, _, _) => smallvec![StyleState::Focused],
+            InnerState::Idle => StateVec::new(),
+        }
+    }
+
     fn len(&self) -> usize {
         0
     }
 
-    fn visit_children(&mut self, _: &mut dyn FnMut(&mut Node<'a, T>)) {}
+    fn visit_children(&mut self, _: &mut dyn FnMut(&mut dyn GenericNode<'a, T>)) {}
 
-    fn size(&self, stylesheet: &Stylesheet) -> (Size, Size) {
+    fn size(&self, _: &State, stylesheet: &Stylesheet) -> (Size, Size) {
         match (stylesheet.width, stylesheet.height) {
             (Size::Shrink, Size::Shrink) => {
-                let width = self.placeholder(stylesheet).measure(None).width()
+                let width = self.placeholder_text(stylesheet).measure(None).width()
                     + stylesheet.padding.left
                     + stylesheet.padding.right;
                 let metrics = stylesheet.font.inner.v_metrics(Scale::uniform(stylesheet.text_size));
@@ -134,7 +197,7 @@ where
             }
 
             (Size::Shrink, other) => {
-                let width = self.placeholder(stylesheet).measure(None).width()
+                let width = self.placeholder_text(stylesheet).measure(None).width()
                     + stylesheet.padding.left
                     + stylesheet.padding.right;
                 (Size::Exact(width), other)
@@ -152,6 +215,7 @@ where
 
     fn event(
         &mut self,
+        state: &mut State,
         layout: Rectangle,
         clip: Rectangle,
         stylesheet: &Stylesheet,
@@ -163,7 +227,7 @@ where
         let mut new_text = None;
 
         // sanity check on the state
-        self.state.inner = match self.state.inner {
+        state.inner = match state.inner {
             InnerState::Dragging(mut from, mut to, since) => {
                 if from > value_len {
                     from = value_len;
@@ -192,42 +256,42 @@ where
         // event related state update
         match event {
             Event::Cursor(x, y) => {
-                self.state.cursor = (x, y);
-                if let InnerState::Dragging(from, _, _) = self.state.inner {
+                state.cursor = (x, y);
+                if let InnerState::Dragging(from, _, _) = state.inner {
                     let relative_cursor = (
-                        self.state.cursor.0 - content_rect.left + self.state.scroll_x,
-                        self.state.cursor.1 - content_rect.top + self.state.scroll_y,
+                        state.cursor.0 - content_rect.left + state.scroll_x,
+                        state.cursor.1 - content_rect.top + state.scroll_y,
                     );
                     let hit =
                         text_display(self.text(stylesheet), self.password).hitdetect(relative_cursor, content_rect);
-                    self.state.inner = InnerState::Dragging(from, hit, Instant::now());
+                    state.inner = InnerState::Dragging(from, hit, Instant::now());
                     context.redraw();
                 }
             }
 
             Event::Modifiers(modifiers) => {
-                self.state.modifiers = modifiers;
+                state.modifiers = modifiers;
             }
 
             Event::Press(Key::LeftMouseButton) => {
                 context.redraw();
-                if layout.point_inside(self.state.cursor.0, self.state.cursor.1)
-                    && clip.point_inside(self.state.cursor.0, self.state.cursor.1)
+                if layout.point_inside(state.cursor.0, state.cursor.1)
+                    && clip.point_inside(state.cursor.0, state.cursor.1)
                 {
                     let relative_cursor = (
-                        self.state.cursor.0 - content_rect.left + self.state.scroll_x,
-                        self.state.cursor.1 - content_rect.top + self.state.scroll_y,
+                        state.cursor.0 - content_rect.left + state.scroll_x,
+                        state.cursor.1 - content_rect.top + state.scroll_y,
                     );
                     let hit =
                         text_display(self.text(stylesheet), self.password).hitdetect(relative_cursor, content_rect);
-                    self.state.inner = InnerState::Dragging(hit, hit, Instant::now());
+                    state.inner = InnerState::Dragging(hit, hit, Instant::now());
                 } else {
-                    self.state.inner = InnerState::Idle;
+                    state.inner = InnerState::Idle;
                 }
             }
 
             Event::Release(Key::LeftMouseButton) => {
-                self.state.inner = match self.state.inner {
+                state.inner = match state.inner {
                     InnerState::Dragging(from, to, since) => {
                         context.redraw();
                         InnerState::Focused(from, to, since)
@@ -236,10 +300,10 @@ where
                 }
             }
 
-            event => match self.state.inner {
+            event => match state.inner {
                 InnerState::Idle => match event {
                     Event::Press(key) if Some(key) == self.trigger => {
-                        self.state.inner = InnerState::Focused(0, self.value.as_ref().len(), Instant::now());
+                        state.inner = InnerState::Focused(0, self.value.as_ref().len(), Instant::now());
                         context.redraw();
                     }
                     _ => (),
@@ -247,25 +311,25 @@ where
 
                 InnerState::Focused(from, to, _) => match event {
                     Event::Text(c) => match c {
-                        '\x08' => {
+                        BACKWARDS_DELETE => {
                             context.redraw();
                             let (from, to) = (from.min(to), from.max(to));
 
                             if to > from {
-                                self.state.inner = InnerState::Focused(from, from, Instant::now());
+                                state.inner = InnerState::Focused(from, from, Instant::now());
                                 let (head, tail) = self.value.as_ref().split_at(codepoint(self.value.as_ref(), from));
                                 new_text.replace(format!("{}{}", head, tail.split_at(codepoint(tail, to - from)).1));
                             } else if from > 0 {
-                                self.state.inner = InnerState::Focused(from - 1, from - 1, Instant::now());
+                                state.inner = InnerState::Focused(from - 1, from - 1, Instant::now());
                                 let (head, tail) =
                                     self.value.as_ref().split_at(codepoint(self.value.as_ref(), from - 1));
                                 new_text.replace(format!("{}{}", head, tail.split_at(codepoint(tail, 1)).1));
                             }
                         }
-                        '\x7f' => {
+                        FORWARD_DELETE => {
                             context.redraw();
                             let (from, to) = (from.min(to), from.max(to));
-                            self.state.inner = InnerState::Focused(from, from, Instant::now());
+                            state.inner = InnerState::Focused(from, from, Instant::now());
 
                             let (head, tail) = self.value.as_ref().split_at(codepoint(self.value.as_ref(), from));
                             if to > from {
@@ -278,7 +342,7 @@ where
                             if !c.is_control() {
                                 context.redraw();
                                 let (from, to) = (from.min(to), from.max(to));
-                                self.state.inner = InnerState::Focused(from + 1, from + 1, Instant::now());
+                                state.inner = InnerState::Focused(from + 1, from + 1, Instant::now());
 
                                 let (head, tail) = self.value.as_ref().split_at(codepoint(self.value.as_ref(), from));
                                 if to > from {
@@ -296,16 +360,16 @@ where
                     },
 
                     Event::Press(Key::Enter) if self.on_submit.is_some() => {
-                        if !self.state.modifiers.shift {
+                        if !state.modifiers.shift {
                             context.redraw();
                             context.extend(self.on_submit.take());
-                            self.state.inner = InnerState::Idle;
+                            state.inner = InnerState::Idle;
                         }
                     }
 
                     #[cfg(feature = "clipboard")]
                     Event::Press(Key::C) => {
-                        if self.state.modifiers.ctrl {
+                        if state.modifiers.command {
                             let (a, b) = (
                                 codepoint(self.value.as_ref(), from.min(to)),
                                 codepoint(self.value.as_ref(), from.max(to)),
@@ -319,7 +383,7 @@ where
 
                     #[cfg(feature = "clipboard")]
                     Event::Press(Key::X) => {
-                        if self.state.modifiers.ctrl {
+                        if state.modifiers.command {
                             context.redraw();
                             let (from, to) = (from.min(to), from.max(to));
                             let (a, b) = (codepoint(self.value.as_ref(), from), codepoint(self.value.as_ref(), to));
@@ -328,7 +392,7 @@ where
                                 .and_then(|mut cc| cc.set_contents(cut_text))
                                 .ok();
 
-                            self.state.inner = InnerState::Focused(from, from, Instant::now());
+                            state.inner = InnerState::Focused(from, from, Instant::now());
                             let (head, tail) = self.value.as_ref().split_at(codepoint(self.value.as_ref(), from));
                             if to > from {
                                 new_text.replace(format!("{}{}", head, tail.split_at(codepoint(tail, to - from)).1));
@@ -340,14 +404,14 @@ where
 
                     #[cfg(feature = "clipboard")]
                     Event::Press(Key::V) => {
-                        if self.state.modifiers.ctrl {
+                        if state.modifiers.command {
                             context.redraw();
                             let (from, to) = (from.min(to), from.max(to));
                             let paste_text = ClipboardContext::new().and_then(|mut cc| cc.get_contents()).ok();
 
                             if let Some(paste_text) = paste_text {
                                 let (head, tail) = self.value.as_ref().split_at(codepoint(self.value.as_ref(), from));
-                                self.state.inner = InnerState::Focused(
+                                state.inner = InnerState::Focused(
                                     from + paste_text.len(),
                                     from + paste_text.len(),
                                     Instant::now(),
@@ -368,48 +432,59 @@ where
 
                     Event::Press(Key::Left) => {
                         context.redraw();
-                        if self.state.modifiers.shift {
-                            self.state.inner =
-                                InnerState::Focused(from, if to > 0 { to - 1 } else { 0 }, Instant::now());
+                        if state.modifiers.command {
+                            if state.modifiers.shift {
+                                state.inner = InnerState::Focused(from, 0, Instant::now());
+                            } else {
+                                state.inner = InnerState::Focused(0, 0, Instant::now());
+                            }
+                        } else if state.modifiers.shift {
+                            state.inner = InnerState::Focused(from, if to > 0 { to - 1 } else { 0 }, Instant::now());
                         } else {
                             let (from, to) = (from.min(to), from.max(to));
                             if from != to || from == 0 {
-                                self.state.inner = InnerState::Focused(from, from, Instant::now());
+                                state.inner = InnerState::Focused(from, from, Instant::now());
                             } else {
-                                self.state.inner = InnerState::Focused(from - 1, from - 1, Instant::now());
+                                state.inner = InnerState::Focused(from - 1, from - 1, Instant::now());
                             }
                         }
                     }
 
                     Event::Press(Key::Right) => {
                         context.redraw();
-                        if self.state.modifiers.shift {
-                            self.state.inner = InnerState::Focused(from, (to + 1).min(value_len), Instant::now());
+                        if state.modifiers.command {
+                            if state.modifiers.shift {
+                                state.inner = InnerState::Focused(from, value_len, Instant::now());
+                            } else {
+                                state.inner = InnerState::Focused(value_len, value_len, Instant::now());
+                            }
+                        } else if state.modifiers.shift {
+                            state.inner = InnerState::Focused(from, (to + 1).min(value_len), Instant::now());
                         } else {
                             let (from, to) = (from.min(to), from.max(to));
                             if from != to || to >= value_len {
-                                self.state.inner = InnerState::Focused(to, to, Instant::now());
+                                state.inner = InnerState::Focused(to, to, Instant::now());
                             } else {
-                                self.state.inner = InnerState::Focused(to + 1, to + 1, Instant::now());
+                                state.inner = InnerState::Focused(to + 1, to + 1, Instant::now());
                             }
                         }
                     }
 
                     Event::Press(Key::Home) => {
                         context.redraw();
-                        if self.state.modifiers.shift {
-                            self.state.inner = InnerState::Focused(from, 0, Instant::now());
+                        if state.modifiers.shift {
+                            state.inner = InnerState::Focused(from, 0, Instant::now());
                         } else {
-                            self.state.inner = InnerState::Focused(0, 0, Instant::now());
+                            state.inner = InnerState::Focused(0, 0, Instant::now());
                         }
                     }
 
                     Event::Press(Key::End) => {
                         context.redraw();
-                        if self.state.modifiers.shift {
-                            self.state.inner = InnerState::Focused(from, value_len, Instant::now());
+                        if state.modifiers.shift {
+                            state.inner = InnerState::Focused(from, value_len, Instant::now());
                         } else {
-                            self.state.inner = InnerState::Focused(value_len, value_len, Instant::now());
+                            state.inner = InnerState::Focused(value_len, value_len, Instant::now());
                         }
                     }
 
@@ -421,10 +496,10 @@ where
         }
 
         // update scroll state for current text and caret position
-        match self.state.inner {
+        match state.inner {
             InnerState::Dragging(_, pos, _) | InnerState::Focused(_, pos, _) => {
                 let mut measure_text = Text {
-                    text: Cow::Borrowed(new_text.as_ref().map(String::as_str).unwrap_or(self.value.as_ref())),
+                    text: Cow::Borrowed(new_text.as_deref().unwrap_or_else(|| self.value.as_ref())),
                     font: stylesheet.font.clone(),
                     size: stylesheet.text_size,
                     wrap: TextWrap::NoWrap,
@@ -439,25 +514,25 @@ where
 
                 let (caret, range) = measure_text.measure_range(pos, measure_text_len, content_rect);
 
-                if self.state.scroll_x + content_rect.width() > range.0 + 2.0 {
+                if state.scroll_x + content_rect.width() > range.0 + 2.0 {
                     context.redraw();
-                    self.state.scroll_x = (range.0 - content_rect.width() + 2.0).max(0.0);
+                    state.scroll_x = (range.0 - content_rect.width() + 2.0).max(0.0);
                 }
-                if caret.0 - self.state.scroll_x > content_rect.width() - 2.0 {
+                if caret.0 - state.scroll_x > content_rect.width() - 2.0 {
                     context.redraw();
-                    self.state.scroll_x = caret.0 - content_rect.width() + 2.0;
+                    state.scroll_x = caret.0 - content_rect.width() + 2.0;
                 }
-                if caret.0 - self.state.scroll_x < 0.0 {
+                if caret.0 - state.scroll_x < 0.0 {
                     context.redraw();
-                    self.state.scroll_x = caret.0;
+                    state.scroll_x = caret.0;
                 }
-                if caret.1 - self.state.scroll_y > content_rect.height() - 2.0 {
+                if caret.1 - state.scroll_y > content_rect.height() - 2.0 {
                     context.redraw();
-                    self.state.scroll_y = caret.1 - content_rect.height() + 2.0;
+                    state.scroll_y = caret.1 - content_rect.height() + 2.0;
                 }
-                if caret.1 - self.state.scroll_y < 0.0 {
+                if caret.1 - state.scroll_y < 0.0 {
                     context.redraw();
-                    self.state.scroll_y = caret.1;
+                    state.scroll_y = caret.1;
                 }
             }
             _ => (),
@@ -468,17 +543,23 @@ where
         }
     }
 
-    fn draw(&mut self, layout: Rectangle, clip: Rectangle, stylesheet: &Stylesheet) -> Vec<Primitive<'a>> {
+    fn draw(
+        &mut self,
+        state: &mut State,
+        layout: Rectangle,
+        clip: Rectangle,
+        stylesheet: &Stylesheet,
+    ) -> Vec<Primitive<'a>> {
         let mut result = Vec::new();
 
         let content_rect = self.content_rect(layout, stylesheet);
-        let text_rect = content_rect.translate(-self.state.scroll_x, -self.state.scroll_y);
+        let text_rect = content_rect.translate(-state.scroll_x, -state.scroll_y);
         let text = text_display(self.text(stylesheet), self.password);
 
         result.extend(stylesheet.background.render(layout).into_iter());
         if let Some(clip) = content_rect.intersect(&clip) {
             result.push(Primitive::PushClip(clip));
-            match self.state.inner {
+            match state.inner {
                 InnerState::Dragging(from, to, since) | InnerState::Focused(from, to, since) => {
                     let range = text.measure_range(from.min(to), from.max(to), text_rect);
 
@@ -521,7 +602,10 @@ where
                 _ => (),
             }
             if self.value.as_ref().is_empty() {
-                result.push(Primitive::DrawText(self.placeholder(stylesheet).to_owned(), text_rect));
+                result.push(Primitive::DrawText(
+                    self.placeholder_text(stylesheet).to_owned(),
+                    text_rect,
+                ));
             } else {
                 result.push(Primitive::DrawText(text, text_rect));
             }
@@ -539,7 +623,7 @@ where
     S: 'a + Send + AsRef<str>,
 {
     fn into_node(self) -> Node<'a, T> {
-        Node::new(self)
+        Node::from_widget(self)
     }
 }
 
@@ -553,6 +637,7 @@ impl Default for State {
                 alt: false,
                 shift: false,
                 logo: false,
+                command: false,
             },
             inner: InnerState::Idle,
             cursor: (0.0, 0.0),
